/*

 * cmdlineprop.h

 * Asa MacWilliams, April 2002

 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de

 *

 * For documentation and instructions, contact me at asa.macwilliams@in.tum.de.

 */



#ifdef WIN32

# pragma warning(disable: 4786)

#endif



#include "cmdlineprops.h"



#include <map>

#include <iostream>

#include <string>



using namespace std;



//////////////////////////

CmdLineProps cmdLineProps;

//////////////////////////



class CmdLinePropsData {

private:

	typedef map<string, string, less<string> > stringmap_t;

	stringmap_t cmdlineMap;

//	stringmap_t environMap;

//	stringmap_t propMap;

	

public:

  void init(int argc, char** argv);

	bool find(const char* name);

	const char* get(const char* name);

	const char* getname(int index);

	int count();

};



//////////////////////////



void CmdLinePropsData::init(int argc, char** argv)

{

	for(int i=0; i<argc; i++)

	{

		const char* arg=argv[i];

		//find "-Dname=value"

		if(arg[0]!='-' || arg[1]!='D')

			continue;

		//find name

		const char *namestart=arg+2;

		const char *equals=namestart;

		while(*equals && *equals!='=')

			equals++;

		const char *namepastend=equals;

		if(equals==namestart)

		{

			cerr << "Properties must be defined as '-Dname[=value]'" << endl;

			continue;

		}

		if(*equals!='=')

		{

			string name(namestart, namepastend-namestart);

			string val("");

			cmdlineMap[name]=val;

			continue;

		}

		//find value

		const char *valstart=equals+1;

		const char *valpastend=valstart;

		while(*valpastend)

			valpastend++;

		//done

		string name(namestart, namepastend-namestart);

		string val(valstart, valpastend-valstart);

		cmdlineMap[name]=val;

		//cerr << "Property '" << name << "' set to '" << val << "'." << endl;

	}

}



bool CmdLinePropsData::find(const char* name)

{

	return cmdlineMap.count(name)!=0;

}



const char* CmdLinePropsData::get(const char* name)

{

	if(cmdlineMap.count(name))

		return cmdlineMap[name].c_str();

	else

		return 0;

}



const char* CmdLinePropsData::getname(int index)

{

	stringmap_t::iterator i;

	for(i=cmdlineMap.begin(); i!=cmdlineMap.end(); i++)

		if(index--<=0)

			return (*i).first.c_str();

	return "";

}



int CmdLinePropsData::count()

{

	return cmdlineMap.size();

}



//////////////////////////



CmdLineProps::CmdLineProps()

{

	data=new CmdLinePropsData;

}



void CmdLineProps::init(int argc, char** argv)

{

	data->init(argc, argv);

}



bool CmdLineProps::find(const char* name)

{

	return data->find(name);

}



const char* CmdLineProps::get(const char* name)

{

	if(find(name))

		return data->get(name);

	else

		return "";

}



const char* CmdLineProps::getname(int index)

{

	if(index>=0 && index<data->count())

		return data->getname(index);

	else

		return "";

}



int CmdLineProps::count()

{

	return data->count();

}



